package com.gemho.environment.repository;

import com.gemho.environment.model.Sensor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;

import java.util.List;

/**
* @author 靖小侠
* @date 2020-05-07
*/
public interface SensorRepository extends JpaRepository<Sensor, Long>, JpaSpecificationExecutor<Sensor> {
    Sensor findByImei(String imei);

    sun.management.Sensor findByDeviceid(String deviceid);

    @Query(value = "select s.* from s_sensor s LEFT JOIN s_user_sensor u ON s.deviceid = u.sensorid where u.userid = ?1 ",
            nativeQuery = true)
    List<Sensor> findSensorByUserid(Long userid);

    @Query(value = "select case WHEN (unix_timestamp()-unix_timestamp(s.onlinetime)) <= ((s.timetrack2)/1000)*2 THEN '在线' ELSE '离线' END online,  s.* from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1",
            countQuery = "select count(1) from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1",
            nativeQuery = true)
    Page<Sensor> queryAllByUserid(Long userid, Pageable pageable);

    @Query(value = "select case WHEN (unix_timestamp()-unix_timestamp(s.onlinetime)) <= ((s.timetrack2)/1000)*2 THEN '在线' ELSE '离线' END online,  s.* from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1 and s.deviceid like ?2",
            countQuery = "select count(1) from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1 and s.deviceid like ?2",
            nativeQuery = true)
    Page<Sensor> queryAllByUseridAndDeviceid(Long userid, String deviceid, Pageable pageable);

    @Query(value = "select case WHEN (unix_timestamp()-unix_timestamp(s.onlinetime)) <= ((s.timetrack2)/1000)*2 THEN '在线' ELSE '离线' END online,  s.* from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1 and s.imei like ?2",
            countQuery = "select count(1) from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1 and s.imei like ?2",
            nativeQuery = true)
    Page<Sensor> queryAllByUseridAndImei(Long userid, String imei, Pageable pageable);

    @Query(value = "select  case WHEN (unix_timestamp()-unix_timestamp(s.onlinetime)) <= ((s.timetrack2)/1000)*2 THEN '在线' ELSE '离线' END online,  s.* from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1 and s.name like ?2",
            countQuery = "select count(1) from s_sensor s left join s_user_sensor sus on sus.sensorid = s.deviceid where sus.userid = ?1 and s.name like ?2",
            nativeQuery = true)
    Page<Sensor> queryAllByUseridAndName(Long userid, String name, Pageable pageable);

    @Query(value = "select su.* from ( " +
            " select s.* from s_sensor s LEFT JOIN s_user_sensor u ON s.deviceid = u.sensorid where u.userid = ?1 " +
            ") su LEFT JOIN s_sensor_device r on su.deviceid = r.sensorid WHERE r.deviceid = ?2 ",
            nativeQuery = true)
    List<Sensor> findSensorByUserAndRelay(Long userid, String relayid);

    @Query(value = "select count(s.id) AS total FROM s_sensor s  INNER JOIN s_user_sensor us ON us.sensorid = s.deviceid where unix_timestamp()-unix_timestamp((s.onlinetime)) <= ((s.timetrack2)/1000)*2 and us.userid = ?1", nativeQuery = true)
    int queryOnlineSensorByUser(Long userid);
    @Query(value = "select count(s.id) AS total FROM s_sensor s  INNER JOIN s_user_sensor us ON us.sensorid = s.deviceid where (unix_timestamp() -unix_timestamp((s.onlinetime)) > ((s.timetrack2)/1000)*2 OR s.onlinetime is NULL) and us.userid = ?1", nativeQuery = true)
    int queryNotOnlineSensorByUser(Long userid);

    @Query(value = "select count(s.id) AS total FROM s_sensor s  where unix_timestamp()-unix_timestamp((s.onlinetime)) <= ((s.timetrack2)/1000)*2 ", nativeQuery = true)
    int queryOnlineSensorBySuper();
    @Query(value = "select count(s.id) AS total FROM s_sensor s  where unix_timestamp()-unix_timestamp((s.onlinetime)) > ((s.timetrack2)/1000)*2 OR s.onlinetime is NULL ", nativeQuery = true)
    int queryNotOnlineSensorBySuper();
}